package com.mycompany.commerce.project.facade.server.services.dataaccess.bom.mediator;
/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2008
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.ibm.commerce.foundation.client.util.oagis.RelationalExpression;
import com.ibm.commerce.foundation.common.exception.AbstractApplicationException;
import com.ibm.commerce.foundation.common.util.logging.LoggingHelper;
import com.ibm.commerce.foundation.server.services.dataaccess.DataServiceFacade;
import com.ibm.commerce.foundation.server.services.dataaccess.PhysicalDataContainer;
import com.ibm.commerce.foundation.server.services.dataaccess.SelectionCriteria;
import com.ibm.commerce.foundation.server.services.dataaccess.bom.mediator.AbstractChangeBusinessObjectPartMediatorImpl;
import com.ibm.commerce.foundation.server.services.dataaccess.exception.DataMediatorException;
import com.mycompany.commerce.project.facade.ProjectFacadeConstants;
import com.mycompany.commerce.project.facade.datatypes.ProjectCollectionReferenceType;
import com.mycompany.commerce.project.facade.datatypes.ProjectType;
import com.mycompany.commerce.project.facade.server.entity.datatypes.Xprjprjcolrel;
import com.mycompany.commerce.project.facade.server.helpers.ProjectComponentHelper;

/**
 * Update operation on the logical noun for Project
 */
public class ChangeProjectCollectionRelPartMediator extends
		AbstractChangeBusinessObjectPartMediatorImpl {

	private static final String CLASSNAME = ChangeProjectCollectionRelPartMediator.class
			.getName();

	private static final Logger LOGGER = LoggingHelper
			.getLogger(ChangeProjectCollectionRelPartMediator.class);

	public static final String CHANGE_PROJECT_PrjColRel_ACCESS_PROFILE = "MyCompany_ProjectPrjColRel_Update";

	private static final String UNIQUE_ID = "UniqueID";

	/**
	 * Initialize the physical data container for this mediator. Obtain the
	 * physical entities that correspond to this logical entity. The data from
	 * noun present in the list will be used to fetch the physical entities with
	 * the help of queries configured with DSL
	 * 
	 * @throws DataMediatorException
	 * @see com.ibm.commerce.foundation.server.services.dataaccess.bom.mediator.AbstractChangeBusinessObjectMediatorImpl#initializePhysicalDataContainer(java.util.List)
	 */
	protected PhysicalDataContainer initializePhysicalDataContainer(
			Map aMapNounPart) throws DataMediatorException {
		final String METHODNAME = "initializePhysicalDataContainer";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME,
					new Object[] { aMapNounPart });
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}

		PhysicalDataContainer physicalEntites = null;

		List projectIds = new ArrayList();
		Iterator projectIter = aMapNounPart.keySet().iterator();
		while (projectIter.hasNext()) {
			ProjectType aProject = (ProjectType) projectIter.next();
			String uniqueId = aProject.getProjectIdentifier().getUniqueID();
			if (uniqueId != null && uniqueId.length() > 0) {
				projectIds.add(uniqueId);
			}
		}

		if (!projectIds.isEmpty()) {
			SelectionCriteria anExpression = new SelectionCriteria();
			anExpression.setXPathKey(ProjectFacadeConstants.XPATH_PROJECT_UID
					+ ProjectFacadeConstants.DOUBLE_CLOSE_XPATH);
			;
			anExpression
					.setAccessProfile(CHANGE_PROJECT_PrjColRel_ACCESS_PROFILE);
			anExpression.setXPathParameter(new RelationalExpression(UNIQUE_ID,
					projectIds));

			physicalEntites = DataServiceFacade.getInstance(
					ProjectFacadeConstants.COMPONENT_NAME)
					.getPhysicalDataContainer(anExpression);

		} else {
			physicalEntites = DataServiceFacade.getInstance(
					ProjectFacadeConstants.COMPONENT_NAME)
					.getEmptyPhysicalDataContainer();

		}

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}
		return physicalEntites;
	}
	/**
	 * This method performs no actions as the resolve is done by the
	 * <code>ChangeProjectMediator</code>.
	 * 
	 * @param aNoun
	 *            The logical <code>ProjectType</code> SDO business object.
	 * @param aNounPart
	 *            The logical <code>ProjectType</code> SDO business object.
	 * @return This method returns false.
	 */
	protected boolean resolveByExternalIdentifier(Object arg0, Object arg1)
			throws Exception {

		return false;
	}

	/**
	 * Checks the object to be created . If there are errors then a validation
	 * error will be added to the return list
	 * 
	 * @param aNoun
	 *            The noun that is being requested to be changed
	 * @param aNounPart
	 *            The noun part that is being requested to be changed
	 * @return List of errors; null if there are no errors
	 * @throws DataMediatorException
	 * 
	 * @see com.ibm.commerce.foundation.server.services.dataaccess.bom.mediator.ChangeBusinessObjectPartMediator#validateCreate(java.lang.Object)
	 */
	public List validateCreate(Object aNoun, Object aNounPart)
			throws DataMediatorException {
		final String METHODNAME = "validateCreate";
		boolean traceEnabled = LoggingHelper.isTraceEnabled(LOGGER);
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] { aNoun,
					aNounPart });
		}

		List validationErrors = new ArrayList();
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME, validationErrors);
		}
		return validationErrors;
	}

	/**
	 * This method checks if the logical <code>ProjectType</code> noun can be
	 * updated. It checks that the noun exists in the physical data container If
	 * the noun cannot be updated, the appropriate error exception is added to
	 * the list of exceptions to be returned to the caller.
	 * 
	 * @param aNoun
	 *            The logical <code>ProjectType</code> SDO business object to
	 *            validate.
	 * @param aNounPart
	 *            The logical <code>ProjectCollectionReferenceType</code> SDO
	 *            business object.
	 * @return This method returns a list of <code>ApplicationError</code> if
	 *         there are any errors, otherwise the list will be empty.
	 * @throws DataMediatorException
	 */
	public List validateChange(Object aNoun, Object aNounPart)
			throws AbstractApplicationException {
		final String METHODNAME = "validateChange";
		boolean traceEnabled = LoggingHelper.isTraceEnabled(LOGGER);
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] { aNoun,
					aNounPart });
		}
		List validationErrors = new ArrayList();
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME, validationErrors);
		}
		return validationErrors;
	}

	/**
	 * This method checks if the logical <code>ProjectType</code> noun can be
	 * deleted. It checks that the noun exists in the physical data container If
	 * the noun cannot be deleted, the appropriate error exception is added to
	 * the list of exceptions to be returned to the caller.
	 * 
	 * @param aNoun
	 *            The logical <code>ProjectType</code> SDO business object to
	 *            validate.
	 * @param aNounPart
	 *            The logical <code>ProjectCollectionReferenceType</code> SDO
	 *            business object.
	 * @return This method returns a list of <code>ApplicationError</code> if
	 *         there are any errors, otherwise the list will be empty.
	 * @throws DataMediatorException
	 */
	public List validateDelete(Object aNoun, Object aNounPart)
			throws AbstractApplicationException {
		final String METHODNAME = "validateDelete";
		boolean traceEnabled = LoggingHelper.isTraceEnabled(LOGGER);
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] { aNoun,
					aNounPart });
		}
		List validationErrors = new ArrayList();
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME, validationErrors);
		}
		return validationErrors;
	}

	/**
	 * Creates the physical entities corresponding to the logical noun part and
	 * adds to the root physical entity
	 * 
	 * @param aNoun
	 *            (ProjectType) The logical noun that contains the noun part
	 * @param aNounPart
	 *            (ProjectCollectionReferenceType) The noun part for which the
	 *            corresponding physical entities needs to be created
	 * @throws AbstractApplicationException
	 *             Error during create
	 */
	public void create(Object aNoun, Object aNounPart)
			throws AbstractApplicationException {
		final String METHODNAME = "create";

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] { aNoun,
					aNounPart });

		}
		delete(aNoun,aNounPart);
		ProjectType aLogicalProject = (ProjectType) aNoun;

		ProjectCollectionReferenceType aLogicalProjectPrjColRel = (ProjectCollectionReferenceType) aNounPart;

		Integer storeId = null;
		if (aLogicalProjectPrjColRel.getStoreIdentifier() != null
				&& aLogicalProjectPrjColRel.getStoreIdentifier().getUniqueID() != null) {
			//Get store id from the noun
			storeId = Integer.valueOf(aLogicalProjectPrjColRel
					.getStoreIdentifier().getUniqueID());
		} else {
			//Get store id from the context
			storeId = ProjectComponentHelper.getStoreId();
		}

		PhysicalDataContainer pdc = getPhysicalDataContainer();

		Integer aProjectId = Integer.valueOf(aLogicalProject
				.getProjectIdentifier().getUniqueID());

		Xprjprjcolrel aPhysicalXprjprjcolrel = (Xprjprjcolrel) getPhysicalDataContainer()
				.createPhysicalObject(null, Xprjprjcolrel.class);

		aPhysicalXprjprjcolrel.setXproject_id(aProjectId.intValue());
		aPhysicalXprjprjcolrel.setXprjcol_id(new Long(aLogicalProjectPrjColRel
				.getProjectCollectionIdentifier().getUniqueID()).longValue());
		aPhysicalXprjprjcolrel.setStoreent_id(storeId.intValue());

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {

			LOGGER.exiting(CLASSNAME, METHODNAME);
		}

	}

	/**
	 * Updates the physical entities corresponding to the logical noun part and
	 * adds to the root physical entity.This method performs no actions as it does not need update services.
	 * 
	 * @param aNoun
	 *            (ProjectType) The logical noun that contains the noun part
	 * @param aNounPart
	 *            (ProjectCollectionReferenceType) The noun part for which the
	 *            corresponding physical entities needs to be created
	 * @throws AbstractApplicationException
	 *             Error during create
	 */
	public void update(Object aNoun, Object aNounPart)
			throws AbstractApplicationException {
		final String METHODNAME = "update";

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] { aNoun,
					aNounPart });
		}
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}

	}

	/**
	 * Deletes the physical entities corresponding to the logical noun part and
	 * adds to the root physical entity
	 * 
	 * @param aNoun
	 *            (ProjectType) The logical noun that contains the noun part
	 * @param aNounPart
	 *            (ProjectCollectionReferenceType) The noun part for which the
	 *            corresponding physical entities needs to be created
	 * @throws AbstractApplicationException
	 *             Error during create
	 */
	public void delete(Object aNoun, Object aNounPart)
			throws AbstractApplicationException {
		final String METHODNAME = "delete";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] { aNoun,
					aNounPart });

		}

		Xprjprjcolrel aPhysicalEntity = (Xprjprjcolrel) findPhysicalEntity(
				aNoun, aNounPart);

		if (aPhysicalEntity != null) {
			getPhysicalDataContainer().removePhysicalObject(aPhysicalEntity);

		}

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {

			LOGGER.exiting(CLASSNAME, METHODNAME);
		}

	}

	/**
	 * This method returns the <code>Xprjcolrel</code> physical SDO from the
	 * <code>PhysicalDataContainer</code> that was created in the
	 * <code>ChangeProjectPrjColRelMediator</code> mediator.
	 * 
	 * @param aNoun
	 *            The logical <code>ProjectType</code> SDO.
	 * @param aNounPart
	 *            The logical <code>ProjectCollectionReferenceType</code> SDO.
	 * @return This method returns the <code>Xprjcolrel</code> object if it is
	 *         found, otherwise it returns null.
	 * @throws DataMediatorException
	 */
	public Object findPhysicalEntity(Object aNoun, Object aNounPart)
			throws AbstractApplicationException {
		final String METHODNAME = "findPhysicalEntity";
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] { aNoun,
					aNounPart });
		}

		Xprjprjcolrel aPhysicalEntity = null;
		PhysicalDataContainer pdc = getPhysicalDataContainer();

		// if the PhysicalDataContainer is not null, loop it and compare the uid
		// to find the DataObject
		if (pdc != null) {
			ProjectType aLogicalProject = (ProjectType) aNoun;
			ProjectCollectionReferenceType aLogicalProjectColRef = (ProjectCollectionReferenceType) aNounPart;

			long colRefId = Long.parseLong(aLogicalProjectColRef
					.getProjectCollectionIdentifier().getUniqueID());

			if (aLogicalProject.getProjectIdentifier().getUniqueID() != null) {
				long projectId = Long.parseLong(aLogicalProject
						.getProjectIdentifier().getUniqueID());
				if (LoggingHelper.isTraceEnabled(LOGGER)) {
					LOGGER.logp(Level.FINE, CLASSNAME, METHODNAME,
							"Find Project: " + projectId);
				}

				List projectList = pdc.getPhysicalObjects();
				if (projectList != null && projectList.size() > 0) {
					Iterator projectIter = projectList.iterator();
					while (projectIter.hasNext()) {
						Xprjprjcolrel tempPhysicalEntity = (Xprjprjcolrel) projectIter
								.next();
						// compare id to get the Physical DataObject
						if (tempPhysicalEntity.getXproject_id() == projectId) {							

							aPhysicalEntity = tempPhysicalEntity;
							break;
						}
					}
				}

			}
		}

		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME, aPhysicalEntity);
		}

		return aPhysicalEntity;
	}

	/**
	 * This method gets the xpath for the main noun part.
	 * 
	 * @return The xpath for the main noun part.
	 */
	public String[] getNounPartXPaths() {

		return XPATH;
	}

	private static final String[] XPATH = new String[] { "/Collection[]" };

}
